@@ -92,6 +92,7 @@ typedef struct _extlist {
 int incext    = 0, stdinput = 0;
 int overwrite = 0, interact = 1;
 int quiet     = 0, decoall  = 0;
+int nobar     = 0;
 
 /*
  * Global Variables
@@ -100,6 +101,16 @@ int quiet     = 0, decoall  = 0;
 extlist *extensions = NULL;
 char savepath[512];
 
+/*
+ * some statistics
+ */
+
+int decodedok     = 0;
+int decodedfailed = 0;
+int decodedmisp   = 0;
+int inputerrors   = 0;
+int inputfiles    = 0;
+
 /*
  * version string
  */
@@ -109,7 +120,7 @@ char uuversion[256] = VERSION "pl" PATCH;
 /*
  * cvs version
  */
-char * uudeview_id = "$Id: uudeview.c,v 1.11 1996/09/20 10:04:38 fp Exp $";
+char * uudeview_id = "$Id: uudeview.c,v 1.13 1996/12/19 23:33:46 fp Exp $";
 
 static int	addexts		_ANSI_ARGS_((char *));
 static int	work_file	_ANSI_ARGS_((char *));
@@ -154,7 +165,7 @@ BusyCallback (void *param, uuprogress *progress)
    * Display progress meter only if stderr is a terminal
    */
 
-  if (!UUISATTY(stderr))
+  if (!UUISATTY(stderr) || nobar)
     return 0;
 
   if ((ptr = _FP_strrchr (progress->curfile, DIRSEPARATOR[0])) == NULL)
@@ -262,16 +273,18 @@ IsHeaderLine (char *text)
  */
 
 static char 
-getyn (char def) 
+getyn (char *theline, char def)
 {
-  char line[20], *ptr=0;
+  char line[256], *ptr=0;
   int isdebug;
 
   if (interact && !feof (stdin)) {
     fflush (stdout);
-    if ((ptr = fgets (line, 19, stdin)) == NULL)
+    if ((ptr = fgets ((theline)?theline:line, 255, stdin)) == NULL) {
+      if (theline) *theline='\0';
       return def;
-    if (line[0]=='4'&&line[1]=='2') {
+    }
+    if (ptr[0]=='4'&&ptr[1]=='2') {
       UUGetOption (UUOPT_DEBUG, &isdebug, NULL, 0);
       UUSetOption (UUOPT_DEBUG, !isdebug, NULL);
       if (!isdebug)
@@ -282,10 +295,13 @@ getyn (char def)
     }
     while (isspace (*ptr) && *ptr!='\n')
       ptr++;
-    if (*ptr == '\n')
+    if (*ptr == '\n') {
+      if (theline) *theline='\0';
       return def;
+    }
   }
   else {
+    if (theline) *theline='\0';
     printf ("%c\n", def);
     return def;
   }
@@ -443,11 +459,13 @@ proc_stdin (void)
   }
   fclose (target);
 
+  inputfiles++;
   if ((res = UULoadFile (stdfile, NULL, 1)) != UURET_OK) {
     fprintf (stderr, "ERROR: while reading from copied standard input %s: %s %s\n",
 	     stdfile, UUstrerror (res),
 	     (res==UURET_IOERR)?
 	     strerror (UUGetOption (UUOPT_ERRNO, NULL, NULL, 0)) : "");
+    inputerrors = 1;
   }
   _FP_free (stdfile);
   stdinput = 1;
@@ -455,6 +473,34 @@ proc_stdin (void)
   return 1;
 }
 
+/*
+ * extract parameters from "command line"
+ */
+
+static void
+makeparams (char *comline)
+{
+  char *argv[32], *p1, *p2;
+  int argc=2;
+
+  if (comline==NULL || *comline=='\0')
+    return;
+
+  argv[1] = p1 = comline;
+
+  while ((p2 = strchr (p1, ' ')) != NULL && argc < 32) {
+    *p2++ = '\0';
+
+    while (*p2 == ' ')
+      p2++;
+    if (*p2 == '\0')
+      break;
+
+    argv[argc++] = p1 = p2;
+  }
+  work_comline (argc, argv);
+}
+
 /*
  * Load a file with command-line parameters (given with @)
  */
@@ -492,21 +538,7 @@ loadparfile (char *filename)
     if (line[0] == '\0' || line[0] == '#')
         continue;
 
-    argc    = 2;
-    argv[1] = p1 = line;
-
-    while ((p2 = strchr (p1, ' ')) != NULL && argc<32) {
-      *p2++ = '\0';
-      
-      while (*p2 == ' ')
-	p2++;
-      if (*p2 == '\0')
-	break;
-
-      argv[argc++] = p1 = p2;
-    }
-
-    work_comline (argc, argv);
+    makeparams (line);
 
     c = fgetc (pfile);
     if (feof (pfile))
@@ -537,6 +569,7 @@ work_comline (int argc, char *argv[])
     if (*argv[number] == '@')
       loadparfile (argv[number]);
     else if (*argv[number] != '-' && *argv[number] != '+') {
+      inputfiles++;
       if ((res = UULoadFile ((*argv[number]=='|')?argv[number]+1:argv[number],
 			     NULL,
 			     (*argv[number]=='|')?1:0)) != UURET_OK) {
@@ -545,6 +578,7 @@ work_comline (int argc, char *argv[])
 		 UUstrerror (res),
 		 (res==UURET_IOERR)?
 		 strerror (UUGetOption (UUOPT_ERRNO, NULL, NULL, 0)) : "");
+	inputerrors++;
       }
     }
     else switch (*(argv[number] + 1)) {
@@ -617,6 +651,9 @@ work_comline (int argc, char *argv[])
     case 't':
       UUSetOption (UUOPT_USETEXT, 1, NULL);
       break;
+    case 'n':
+      nobar = 1;
+      break;
     case 'V':
       fprintf (stdout, "uudeview %spl%s compiled on %s\n",
 	       VERSION, PATCH, __DATE__);
@@ -624,7 +661,7 @@ work_comline (int argc, char *argv[])
       break;
     default:
       usage (argv[0]);
-      exit (1);
+      exit (99);
       break;
     }
   }
@@ -655,7 +692,7 @@ moreCB (void *param, char *string)
     data->aline = 0;
     if (interact) {
       fprintf (stdout, "<<MORE -- (q) to quit>>");
-      if (getyn ('y') == 'q') {
+      if (getyn (NULL, 'y') == 'q') {
 	data->quitit = 1;
 	return 1;
       }
@@ -715,7 +752,7 @@ more (uulist *uin, int type)
 	       UUstrerror(res));
       return;
     }
-    if (UUISATTY(stderr)) {
+    if (UUISATTY(stderr) && !nobar) {
       fprintf (stderr, "%70s\r", ""); /* clear progress information */
       fflush  (stderr);
     }
@@ -744,7 +781,7 @@ more (uulist *uin, int type)
 
   if (interact && data.aline+2 >= data.lines) {
     fprintf (stdout, "<<END -- hit return>>");
-    getyn   ('?');
+    getyn   (NULL, '?');
   }
   else
     fprintf (stdout, "<<END>>");
@@ -762,7 +799,7 @@ DumpFileList (void)
       continue;
     }
 
-    if (UUISATTY(stderr)) {
+    if (UUISATTY(stderr) && !nobar) {
       fprintf (stderr, "\r%70s\r", "");
       fflush  (stderr);
     }
@@ -831,9 +868,15 @@ process_one (uulist *iter)
     }
     
     if ((!quiet || interact) && !decoall)
-      tmp = getyn ('d');
-    else
+      tmp = getyn (line, 'd');
+    else {
+      line[0] = '\0';
       tmp = 'd';
+    }
+    for (ptr1=line; *ptr1 && *ptr1 != tmp; ptr1++);
+    if (*ptr1++==tmp) {
+      while (isspace(*ptr1)) ptr1++;
+    }
 
     if (tmp == 'n')
       return 1;
@@ -894,22 +937,35 @@ process_one (uulist *iter)
       continue;
     }
     else if (tmp == 'r') {
-      printf ("Enter new filename: "); fflush (stdout);
-      if (fgets (line, 250, stdin) == NULL)
-	printf ("\nERROR: Could not get filename: %s\n", strerror (errno));
-      else if (strlen (line) > 1) {
-	line[strlen(line)-1] = '\0';
-	UURenameFile (iter, line);
+      if (strlen(ptr1) <= 1) {
+	printf ("Enter new filename: "); fflush (stdout);
+	if (fgets (line, 250, stdin) == NULL) {
+	  printf ("\nERROR: Could not get filename: %s\n",
+		  strerror (errno));
+	  line[0] = '\0';
+	}
+	ptr1 = line;
+      }
+      if (strlen (ptr1) > 1) {
+	if (ptr1[strlen(ptr1)-1] == '\n')
+	  ptr1[strlen(ptr1)-1] = '\0';
+	UURenameFile (iter, ptr1);
       }
       continue;
     }
     else if (tmp == 'p') {
-      printf ("Enter new path: "); fflush (stdout);
-      if (fgets (line, 250, stdin) == NULL)
-        printf ("\nERROR: Could not get path: %s\n", strerror (errno));
-      else if (strlen (line) > 1) {
-	line[strlen(line)-1] = '\0'; /* remove LF */
-	strcpy (savepath, line);
+      if (strlen(ptr1) <= 1) {
+	printf ("Enter new path: "); fflush (stdout);
+	if (fgets (line, 250, stdin) == NULL) {
+	  printf ("\nERROR: Could not get path: %s\n", strerror (errno));
+	  line[0] = '\0';
+	}
+	ptr1 = line;
+      }
+      if (strlen (ptr1) > 1) {
+	if (ptr1[strlen(ptr1)-1] == '\n')
+	  ptr1[strlen(ptr1)-1] = '\0';
+	strcpy (savepath, ptr1);
 	if (strlen (savepath)) {
 	  if (savepath[strlen(savepath)-1]!=DIRSEPARATOR[0])
 	    strcat (savepath, DIRSEPARATOR);
@@ -932,15 +988,17 @@ process_one (uulist *iter)
 	       (iter->filename) ? iter->filename : "",
 	       (iter->subfname) ? iter->subfname : "",
 	       UUstrerror(res));
+      decodedfailed++;
       break;
     }
-    if (!quiet && UUISATTY(stderr)) {
+    if (!quiet && UUISATTY(stderr) && !nobar) {
       fprintf (stderr, "%70s\r", ""); /* clear progress information */
       fflush  (stderr);
     }
     if (iter->binfile == NULL) {
       fprintf (stderr, "ERROR: Ooops. Decoded %s but no binfile??\n",
 	       (iter->filename) ? iter->filename : "");
+      decodedfailed++;
       break;
     }
     
@@ -964,18 +1022,36 @@ process_one (uulist *iter)
 	while (exists (targetname) && !overwrite && res != 'o') {
 	  printf ("*** (O)verwrite, Overwrite (A)ll, (R)ename, (N)ext, (P)ath [o] ");
 	  fflush (stdout);
-	  switch (res = getyn ('o')) {
+	  /*
+	   * Ask the user for her/his choice. If 'r' or 'p', and there
+	   * is more data on the line, use it as reply
+	   */
+
+	  res = getyn (line, 'o');
+
+	  for (ptr1=line; *ptr1 && *ptr1 != res; ptr1++);
+	  if (*ptr1++==res) {
+	    while (isspace(*ptr1)) ptr1++;
+	  }
+
+	  switch (res) {
 	  case 'n': return 1;
 	  case 'a': overwrite=1; break;
 	  case 'o': break;
 	  case 'r': 
-	    printf ("Enter new filename: "); fflush (stdout);
-	    if (fgets (line, 250, stdin) == NULL)
-	      printf ("\nERROR: Could not get filename: %s\n",
-		      strerror (errno));
-	    else if (strlen (line) > 1) {
-	      line[strlen(line)-1] = '\0';
-	      UURenameFile (iter, line);
+	    if (strlen(ptr1) <= 1) {
+	      printf ("Enter new filename: "); fflush (stdout);
+	      if (fgets (line, 250, stdin) == NULL) {
+		printf ("\nERROR: Could not get filename: %s\n",
+			strerror (errno));
+		line[0] = '\0';
+	      }
+	      ptr1 = line;
+	    }
+	    if (strlen (ptr1) > 1) {
+	      if (ptr1[strlen(ptr1)-1] == '\n')
+		ptr1[strlen(ptr1)-1] = '\0';
+	      UURenameFile (iter, ptr1);
 	    }
 	    if ((name = UUFNameFilter (iter->filename)) == NULL) {
 	      fprintf (stderr, "ERROR: couldn't get filename of %s (%s)\n",
@@ -985,12 +1061,19 @@ process_one (uulist *iter)
 	    }
 	    break;
 	  case 'p':
-	    printf ("Enter new path: "); fflush (stdout);
-	    if (fgets (line, 250, stdin) == NULL)
-	      printf ("\nERROR: Could not get path: %s\n", strerror (errno));
-	    else if (strlen (line) > 1) {
-	      line[strlen(line)-1] = '\0';
-	      strcpy (savepath, line);
+	    if (strlen(ptr1) <= 1) {
+	      printf ("Enter new path: "); fflush (stdout);
+	      if (fgets (line, 250, stdin) == NULL) {
+		printf ("\nERROR: Could not get path: %s\n",
+			strerror (errno));
+		line[0] = '\0';
+	      }
+	      ptr1 = line;
+	    }
+	    if (strlen (ptr1) > 1) {
+	      if (ptr1[strlen(ptr1)-1] == '\n')
+		ptr1[strlen(ptr1)-1] = '\0';
+	      strcpy (savepath, ptr1);
 	      if (strlen(savepath)) {
 		if (savepath[strlen(savepath)-1]!=DIRSEPARATOR[0])
 		  strcat (savepath, DIRSEPARATOR);
@@ -1012,18 +1095,20 @@ process_one (uulist *iter)
 	}
       }
       if ((res = UUDecodeFile (iter, targetname)) != UURET_OK) {
-	if (UUISATTY(stderr))
+	if (UUISATTY(stderr) && !nobar)
 	  fprintf (stderr, "%70s\r", ""); /* clear progress information */
 	fprintf (stderr, "ERROR: while writing %s (%s): %s\n",
 		 targetname, (iter->subfname) ? iter->subfname : "",
 		 UUstrerror(res));
+	decodedfailed++;
 	break;
       }
       if (!quiet) {
-	if (UUISATTY(stderr))
+	if (UUISATTY(stderr) && !nobar)
 	  fprintf (stderr, "%70s\r", ""); /* clear progress information */
 	printf  ("    File successfully written to %s\n", targetname);
       }
+      decodedok++;
       break;
     }
     else if (tmp == 'l') {
@@ -1128,6 +1213,7 @@ process_files (void)
                 (iter->filename) ? iter->filename : "",
 		(iter->subfname) ? iter->subfname : "",
 		ptr, iter->state);
+      decodedmisp++;
       res  = 1;
     }
 
@@ -1156,7 +1242,7 @@ sighandler (int signo)
   printf ("\nReceived Signal (%d), cleaning up temp files.\n", signo);
 
   UUCleanUp ();
-  exit (signo);
+  exit (99);
 }
 
 /*
@@ -1251,7 +1337,14 @@ main (int argc, char *argv[])
   }
   else if (argc < 2) {
     usage (argv[0]);
-    return 1;
+    return 99;
+  }
+
+  /*
+   * use options from UUDEVIEW environment variable
+   */
+  if ((getenv ("UUDEVIEW")) != NULL) {
+    makeparams (getenv ("UUDEVIEW"));
   }
 
   /*
@@ -1261,7 +1354,7 @@ main (int argc, char *argv[])
 
   if (UUInitialize () != UURET_OK) {
     fprintf (stderr, "oops: could not initialize decoding library\n");
-    return 2;
+    return 99;
   }
   if (argc < 2) {
     /*
@@ -1294,7 +1387,7 @@ main (int argc, char *argv[])
   /*
    * clear info
    */
-  if (UUISATTY(stderr)) {
+  if (UUISATTY(stderr) && !nobar) {
     fprintf (stderr, "\r%70s\r", "");
     fflush  (stderr);
   }
@@ -1320,6 +1413,22 @@ Select File-Exit to close window\n\
   }
 #endif  
 
+  if (UUGetOption (UUOPT_VERBOSE, NULL, NULL, 0) && !quiet &&
+      inputfiles && (decodedok || decodedfailed || decodedmisp)) {
+    printf ("%d file%s decoded from %d input file%s, %d failed\n",
+	    decodedok, (decodedok==1)?"":"s",
+	    inputfiles, (inputfiles==1)?"":"s",
+	    decodedfailed+decodedmisp);
+  }
+  
+  /*
+   * determine return value
+   */
+
+  if (decodedfailed || decodedmisp) {
+    return ((decodedok?1:0) | ((decodedmisp||decodedfailed)?2:0));
+  }
+
   return 0;
 }
 
