@@ -54,7 +54,7 @@
 #include <fptools.h>
 #include <uustring.h>
 
-char * uucheck_id = "$Id: uucheck.c,v 1.8 1996/09/20 10:06:57 fp Exp $";
+char * uucheck_id = "$Id: uucheck.c,v 1.10 1996/11/03 12:47:05 fp Exp $";
 
 /*
  * Arbitrary number. This is the maximum number of part numbers we
@@ -75,7 +75,7 @@ static int	UUGetPartNo	_ANSI_ARGS_((char *, char **, char **));
  * State of Scanner function and PreProcessPart
  */
 
-int lastvalid, nofnum;
+int lastvalid, lastenc, nofnum;
 char *uucheck_lastname;
 char *uucheck_tempname;
 static int  lastpart = 0;
@@ -700,6 +700,8 @@ UUPreProcessPart (fileread *data, int *ret)
     *ret = UURET_NOMEM;
     return NULL;
   }
+  memset (result, 0, sizeof (uufile));
+
   if (data->partno) {
     where = whend  = NULL;
     result->partno = data->partno;
@@ -710,7 +712,7 @@ UUPreProcessPart (fileread *data, int *ret)
   }
   else if ((result->partno=UUGetPartNo(data->subject,&where,&whend)) == -2) {
     *ret = UURET_NODATA;
-    _FP_free (result);
+    UUkillfile (result);
     return NULL;
   }
 
@@ -720,7 +722,7 @@ UUPreProcessPart (fileread *data, int *ret)
 		 uustring (S_OUT_OF_MEMORY),
 		 strlen (data->filename)+1);
       *ret = UURET_NOMEM;
-      _FP_free (result);
+      UUkillfile (result);
       return NULL;
     }
   }
@@ -757,9 +759,7 @@ UUPreProcessPart (fileread *data, int *ret)
 		 (result->filename==NULL)?
 		 (strlen(temp)+1):(strlen(result->filename)+1));
       *ret = UURET_NOMEM;
-      _FP_free (result->filename);
-      _FP_free (result->subfname);
-      _FP_free (result);
+      UUkillfile(result);
       return NULL;
     }
     if (result->partno == -1)
@@ -791,8 +791,7 @@ UUPreProcessPart (fileread *data, int *ret)
 		 (result->filename)?
 		 (strlen(result->filename)+1):(strlen(temp)+1));
       *ret = UURET_NOMEM;
-      _FP_free (result->filename);
-      _FP_free (result);
+      UUkillfile (result);
       return NULL;
     }
     /*
@@ -804,6 +803,7 @@ UUPreProcessPart (fileread *data, int *ret)
        * initialize lastvalid
        */
       lastvalid = 1;
+      lastenc   = data->uudet;
       lastpart  = result->partno = 1;
       _FP_strncpy (uucheck_lastname, result->subfname, 256);
     }
@@ -823,22 +823,12 @@ UUPreProcessPart (fileread *data, int *ret)
   }
   else if (result->subfname == NULL && data->uudet) {
     /*
-     * if we have lastvalid, use it
+     * ff we have lastvalid, use it. Make an exception for
+     * Base64-encoded files.
      */
-    if (lastvalid) {
-      result->subfname = _FP_strdup (uucheck_lastname);
-      result->partno   = ++lastpart;
-
+    if (data->uudet == B64ENCODED) {
       /*
-       * if it's the last part, invalidate lastvalid
-       */
-      if (data->end || (data->partno && data->partno == data->maxpno))
-	lastvalid = 0;
-    }
-    else if (data->uudet == B64ENCODED) {
-      /*
-       * exception for MIME-encoded files. Assume it's the first
-       * part. I wonder why it's got no part number?
+       * Assume it's the first part. I wonder why it's got no part number?
        */
       if (result->filename != NULL)
         result->subfname = _FP_strdup (result->filename);
@@ -852,13 +842,20 @@ UUPreProcessPart (fileread *data, int *ret)
 		   (result->filename)?
 		   (strlen(result->filename)+1):(strlen(temp)+1));
 	*ret = UURET_NOMEM;
-        _FP_free (result->filename);
-        _FP_free (result);
+	UUkillfile (result);
         return NULL;
       }
-      lastvalid = 1;
-      lastpart  = result->partno = 1;
-      _FP_strncpy (uucheck_lastname, result->subfname, 256);
+      lastvalid = 0;
+    }
+    else if (lastvalid && data->uudet == lastenc) {
+      result->subfname = _FP_strdup (uucheck_lastname);
+      result->partno   = ++lastpart;
+
+      /*
+       * if it's the last part, invalidate lastvalid
+       */
+      if (data->end || (data->partno && data->partno == data->maxpno))
+	lastvalid = 0;
     }
     else { 
       /* 
@@ -866,8 +863,7 @@ UUPreProcessPart (fileread *data, int *ret)
        * about this part. Let's forget all about it.
        */
       *ret = UURET_NODATA;
-      _FP_free (result->filename);
-      _FP_free (result);
+      UUkillfile (result);
       return NULL;
     }
   }
@@ -877,8 +873,7 @@ UUPreProcessPart (fileread *data, int *ret)
      * should forget about.
      */
     *ret = UURET_NODATA;
-    _FP_free (result->filename);
-    _FP_free (result);
+    UUkillfile (result);
     return NULL;
   }
   else if (result->subfname == NULL) {
@@ -888,8 +883,7 @@ UUPreProcessPart (fileread *data, int *ret)
      * but we don't care here. Just forget it.
      */
     *ret = UURET_NODATA;
-    _FP_free (result->filename);
-    _FP_free (result);
+    UUkillfile (result);
     return NULL;
   }
 
@@ -906,6 +900,7 @@ UUPreProcessPart (fileread *data, int *ret)
     if (!data->end) {
       _FP_strncpy (uucheck_lastname, result->subfname, 256);
       result->partno = lastpart = 1;
+      lastenc = data->uudet;
       lastvalid = 1;
     }
     else
@@ -935,9 +930,7 @@ UUPreProcessPart (fileread *data, int *ret)
      * it's got no data, so why should we need this one anyway?
      */
     *ret = UURET_NODATA;
-    _FP_free (result->subfname);
-    _FP_free (result->filename);
-    _FP_free (result);
+    UUkillfile (result);
     return NULL;
   }
 
@@ -947,9 +940,7 @@ UUPreProcessPart (fileread *data, int *ret)
    */
   if (result->subfname == NULL || result->partno == -1) {
     *ret = UURET_NODATA;
-    _FP_free (result->subfname);
-    _FP_free (result->filename);
-    _FP_free (result);
+    UUkillfile (result);
     return NULL;
   }
 
