@@ -81,7 +81,7 @@
 #include <fptools.h>
 #include <uustring.h>
 
-char * uulib_id = "$Id: uulib.c,v 1.15 1996/09/20 10:50:36 fp Exp $";
+char * uulib_id = "$Id: uulib.c,v 1.16 1996/11/03 12:49:07 fp Exp $";
 
 #ifdef SYSTEM_WINDLL
 BOOL _export WINAPI 
@@ -373,6 +373,7 @@ UUInitialize (void)
   nofnum    = 0;
   mimseqno  = 0;
   lastvalid = 0;
+  lastenc   = 0;
   uuyctr    = 0;
 
   /*
@@ -695,12 +696,12 @@ UULoadFile (char *filename, char *fileid, int delflag)
   if (fileid == NULL)
     fileid = filename;
 
-  while (!feof (datei)) {
+  while (!feof (datei) && !ferror (datei)) {
     /* 
      * Peek file, or some systems won't detect EOF
      */
     res = fgetc (datei);
-    if (feof (datei))
+    if (feof (datei) || ferror (datei))
       break;
     else
       ungetc (res, datei);
