@@ -54,7 +54,7 @@
 #include <fptools.h>
 #include <uustring.h>
 
-char * uunconc_id = "$Id: uunconc.c,v 1.16 1996/09/20 10:51:09 fp Exp $";
+char * uunconc_id = "$Id: uunconc.c,v 1.22 1997/01/16 20:33:12 fp Exp $";
 
 /* for braindead systems */
 #ifndef SEEK_SET
@@ -259,8 +259,8 @@ UUNetscapeCollapse (char *string)
 
   while (*p1) {
     if (*p1 == '<') {
-      if ((_FP_strnicmp (p1, "<ahref=\"", 8) == 0 ||
-	   _FP_strnicmp (p1, "<a href=\"",9) == 0) && 
+      if ((_FP_strnicmp (p1, "<ahref=", 7) == 0 ||
+	   _FP_strnicmp (p1, "<a href=",8) == 0) && 
 	  (_FP_strstr (p1, "</a>") != 0 || _FP_strstr (p1, "</A>") != 0)) {
 	while (*p1 && *p1!='>')        p1++;
 	if (*p1=='\0' || *(p1+1)!='<') return 0;
@@ -435,6 +435,7 @@ UUValidData (char *ptr, int encoding, int *bhflag)
       break;
     }
   }
+
   /*
    * some encoders are broken with respect to encoding the last line of
    * a file and produce extraoneous characters beyond the expected EOL
@@ -518,6 +519,76 @@ UUValidData (char *ptr, int encoding, int *bhflag)
   return XX_ENCODED;        /* data is valid */
 }
 
+/*
+ * This function may be called upon a line that does not look like
+ * valid encoding on first sight, but might be erroneously encoded
+ * data from Netscape, Lynx or MS Exchange. We might need to read
+ * a new line from the stream, which is why we need the FILE.
+ * Returns the type of encoded data if successful or 0 otherwise.
+ */
+
+int
+UURepairData (FILE *datei, char *line, int encoding, int *bhflag)
+{
+  int nflag, vflag=0, safety=42;
+  char *ptr;
+
+  nflag = UUBrokenByNetscape (line);
+
+  while (vflag == 0 && nflag && safety--) {
+    if (nflag == 1) {		/* need next line to repair */
+      ptr = line + strlen (line);
+      while (ptr>line && (*(ptr-1)=='\015' || *(ptr-1)=='\012'))
+	ptr--;
+      if (_FP_fgets (ptr, 255-(ptr-line), datei) == NULL)
+	break;
+    }
+    else {			/* don't need next line to repair */
+    }
+    if (UUNetscapeCollapse (line)) {
+      if ((vflag = UUValidData (line, encoding, bhflag)) == 0)
+	nflag = UUBrokenByNetscape (line);
+    }
+    else
+      nflag = 0;
+  }
+  /*
+   * Sometimes, a line is garbled even without it being split into
+   * the next line. Then we try this in our despair
+   */
+  if (vflag == 0) {
+    if (UUNetscapeCollapse (line))
+      vflag = UUValidData (line, encoding, bhflag);
+  }
+
+  /*
+   * If this line looks uuencoded, but the line is one character short
+   * of a valid line, it was probably broken by MS Exchange. According
+   * to my test cases, there is at most one space character missing;
+   * there are never two spaces together.
+   * If adding a space character helps making this line uuencoded, do
+   * it!
+   */
+
+  if (vflag == 0) {
+    ptr    = line + strlen(line);
+    while (ptr>line && (*(ptr-1)=='\012' || *(ptr-1)=='\015')) {
+      ptr--;
+    }
+    *ptr++ = ' ';
+    *ptr-- = '\0';
+    if ((vflag = UUValidData (line, encoding, bhflag)) != UU_ENCODED) {
+      *ptr  = '\0';
+      vflag = 0;
+    }
+  }
+  return vflag;
+}
+
+/*
+ * Decode a single encoded line using method
+ */
+
 size_t
 UUDecodeLine (char *s, char *d, int method)
 {
@@ -809,11 +880,11 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 	      long maxpos, int method, int flags,
 	      char *boundary)
 {
-  char *line=uugen_fnbuffer, *oline=uuncdp_oline, *ptr;
-  int warning=0, nflag, vlc=0, lc[2], hadct=0;
+  char *line=uugen_fnbuffer, *oline=uuncdp_oline;
+  int warning=0, vlc=0, lc[2], hadct=0;
   int tc=0, tf=0, vflag, haddata=0, haddh=0;
   static int bhflag=0;
-  size_t count;
+  size_t count=0;
 
   if (datain == NULL || dataout == NULL) {
     bhflag = 0;
@@ -943,47 +1014,22 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
       }
     }
     if (*state == DATA || *state == END) {
-      /*
-       * Netscape-Repair code is about the same as in uuio.c
-       */
-      if ((vflag = UUValidData (line, (tf)?method:0, &bhflag)) == 0) {
-	nflag = UUBrokenByNetscape (line);
-
-	while (vflag == 0 && nflag) {
-	  if (nflag == 1) {                   /* need next line to repair */
-	    ptr = line + strlen (line);
-	    while (ptr>line && (*(ptr-1)=='\015' || *(ptr-1)=='\012'))
-	      ptr--;
-	    if (_FP_fgets (ptr, 255-(ptr-line), datain) == NULL)
-	      break;
-	    if (ferror (datain)) {
-	      uu_errno = errno;
-	      return UURET_IOERR;
-	    }
-	  }
-	  else {                       /* don't need next line to repair */
-	  }
-	  if (UUNetscapeCollapse (line)) {
-	    if ((vflag = UUValidData (line, (tf)?method:0, &bhflag)) == 0)
-	      nflag = UUBrokenByNetscape (line);
-	  }
-	  else
-	    nflag = 0;
-	}
-	/*
-	 * Sometimes, a line is garbled even without it being split into
-	 * the next line. Then we try this in our despair
-	 */
-	if (vflag == 0) {
-	  if (UUNetscapeCollapse (line))
-	    vflag = UUValidData (line, (tf)?method:0, &bhflag);
-	}
+      if (method==B64ENCODED && line[0]=='-' && line[1]=='-' && tc) {
+	break;
       }
+
+      if ((vflag = UUValidData (line, (tf)?method:0, &bhflag)) == 0)
+	vflag = UURepairData (datain, line, (tf)?method:0, &bhflag);
+
       /*
-       * correct XXencoded lines that were declared Base64
+       * correct XX/UUencoded lines that were declared Base64
        */
-      if (method == XX_ENCODED && vflag == B64ENCODED)
-	vflag = XX_ENCODED;
+
+      if ((method == XX_ENCODED || method == UU_ENCODED) &&
+	  vflag == B64ENCODED) {
+	if (UUValidData (line, method, &bhflag) == method)
+	  vflag = method;
+      }
 
       if (vflag == method) {
 	if (tf) {
@@ -1024,9 +1070,6 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 	else if (method == BH_ENCODED)
 	  *state = (!line[0] || strchr(line+1,':')==NULL)?DATA:DONE;
       }
-      else if (method==B64ENCODED && line[0]=='-' && line[1]=='-' && tc) {
-	break;
-      }
       else {
 	vlc = tf = tc = 0;
 	haddh = 0;
@@ -1261,8 +1304,7 @@ UUDecode (uulist *data)
     if (res == UURET_OK && state != DONE)
       res = UURET_NOEND;
   }
-
-  if (res != UURET_OK) {
+  else if (res != UURET_OK) {
     data->state &= ~UUFILE_DECODED;
     data->state |=  UUFILE_ERROR | UUFILE_TMPFILE;
   }
@@ -1425,28 +1467,27 @@ int
 UUQuickDecode (FILE *datain, FILE *dataout, char *boundary, long maxpos)
 {
   int state=BEGIN, encoding=-1;
-  char *line=uugen_fnbuffer, *ptr;
-  headers localenv;
+  headers myenv;
 
   /*
    * Read header and find out about encoding.
    */
 
-  memset (&localenv, 0, sizeof (headers));
-  UUScanHeader (datain, &localenv);
+  memset (&myenv, 0, sizeof (headers));
+  UUScanHeader (datain, &myenv);
 
-  if (_FP_stristr (localenv.ctenc, "uu") != NULL)
+  if (_FP_stristr (myenv.ctenc, "uu") != NULL)
     encoding = UU_ENCODED;
-  else if (_FP_stristr (localenv.ctenc, "xx") != NULL)
+  else if (_FP_stristr (myenv.ctenc, "xx") != NULL)
     encoding = XX_ENCODED;
-  else if (_FP_stricmp (localenv.ctenc, "base64") == 0)
+  else if (_FP_stricmp (myenv.ctenc, "base64") == 0)
     encoding = B64ENCODED;
-  else if (_FP_stricmp (localenv.ctenc, "quoted-printable") == 0)
+  else if (_FP_stricmp (myenv.ctenc, "quoted-printable") == 0)
     encoding = QP_ENCODED;
   else
     encoding = PT_ENCODED;
 
-  UUkillheaders (&localenv);
+  UUkillheaders (&myenv);
 
   /*
    * okay, so decode this one
