@@ -57,7 +57,7 @@
 #include <fptools.h>
 #include <uustring.h>
 
-char * uuscan_id = "$Id: uuscan.c,v 1.18 1996/09/28 11:32:21 fp Exp $";
+char * uuscan_id = "$Id: uuscan.c,v 1.23 1997/01/16 20:33:15 fp Exp $";
 
 /*
  * Header fields we recognize as such. See RFC822. We add "From ",
@@ -278,7 +278,10 @@ ParseValue (char *attribute)
   if (attribute == NULL)
     return NULL;
 
-  while (*attribute && *attribute != '=')
+  while ((isalnum(*attribute) || *attribute=='_') && *attribute != '=')
+    attribute++;
+
+  while (isspace(*attribute))
     attribute++;
 
   if (*attribute == '=') {
@@ -286,7 +289,7 @@ ParseValue (char *attribute)
     while (isspace (*attribute))
       attribute++;
   }
-  if (!*attribute)
+  else
     return NULL;
 
   if (*attribute == '"') {
@@ -329,7 +332,7 @@ static headers *
 ParseHeader (headers *theheaders, char *line)
 {
   char **variable=NULL;
-  char *value, *ptr;
+  char *value, *ptr, *thenew;
   int delimit, length;
 
   if (line == NULL)
@@ -373,22 +376,32 @@ ParseHeader (headers *theheaders, char *line)
 
     /* we can probably extract more information */
     if ((ptr = _FP_stristr (line, "boundary")) != NULL) {
-      if (theheaders->boundary) free (theheaders->boundary);
-      theheaders->boundary = _FP_strdup (ParseValue (ptr));
+      if ((thenew = ParseValue (ptr))) {
+	if (theheaders->boundary) free (theheaders->boundary);
+	theheaders->boundary = _FP_strdup (thenew);
+      }
     }
     if ((ptr = _FP_stristr (line, "name")) != NULL) {
-      if (theheaders->fname) free (theheaders->fname);
-      theheaders->fname = _FP_strdup (ParseValue (ptr));
+      if ((thenew = ParseValue (ptr))) {
+	if (theheaders->fname) free (theheaders->fname);
+	theheaders->fname = _FP_strdup (thenew);
+      }
     }
     if ((ptr = _FP_stristr (line, "id")) != NULL) {
-      if (theheaders->mimeid) free (theheaders->mimeid);
-      theheaders->mimeid = _FP_strdup (ParseValue (ptr));
+      if ((thenew = ParseValue (ptr))) {
+	if (theheaders->mimeid) free (theheaders->mimeid);
+	theheaders->mimeid = _FP_strdup (thenew);
+      }
     }
     if ((ptr = _FP_stristr (line, "number")) != NULL) {
-      theheaders->partno = atoi (ParseValue (ptr));
+      if ((thenew = ParseValue (ptr))) {
+	theheaders->partno = atoi (thenew);
+      }
     }
     if ((ptr = _FP_stristr (line, "total")) != NULL) {
-      theheaders->numparts = atoi (ParseValue (ptr));
+      if ((thenew = ParseValue (ptr))) {
+	theheaders->numparts = atoi (thenew);
+      }
     }
   }
   else if (_FP_strnicmp (line, "Content-Transfer-Encoding:", 26) == 0) {
@@ -405,8 +418,9 @@ ParseHeader (headers *theheaders, char *line)
      * TODO: Read RFC1806
      */
     if ((ptr = _FP_stristr (line, "name")) != NULL) {
-      if (theheaders->fname == NULL)
-	theheaders->fname = _FP_strdup (ParseValue (ptr));
+      if (theheaders->fname == NULL && (thenew=ParseValue(ptr)) != NULL) {
+	theheaders->fname = _FP_strdup (thenew);
+      }
     }
     variable = NULL;
   }
@@ -508,7 +522,7 @@ ScanData (FILE *datei, char *fname, int *errcode,
 {
   char *line=uuscan_sdline, *bhds1=uuscan_sdbhds1, *bhds2=uuscan_sdbhds2;
   static char *ptr, *p2, *p3=NULL, *bhdsp, bhl;
-  int bhflag=0, vflag, nflag, haddh=0, hadct=0;
+  int bhflag=0, vflag, haddh=0, hadct=0;
   int bhrpc=0, bhnf=0, c, hcount, lcount, blen;
   int encoding=0, dflag=0, ctline=42;
   int dontcare=0, hadnl=0;
@@ -578,7 +592,7 @@ ScanData (FILE *datei, char *fname, int *errcode,
 	  fseek (datei, preheaders, SEEK_SET);
 	  break;
 	}
-	continue;
+	/* continue; */
       }
       else if (lcount > WAITHEADER) {
 	hcount = 0;
@@ -635,6 +649,10 @@ ScanData (FILE *datei, char *fname, int *errcode,
 	result->mode = result->mode * 8 + *ptr++ - '0';
       while (*ptr == ' ') ptr++;
 
+      /*
+       * We may have picked up a filename from a uuenview-style header
+       */
+      _FP_free (result->filename);
       result->filename = _FP_strdup (ptr);
       result->begin    = 1;
 
@@ -691,10 +709,12 @@ ScanData (FILE *datei, char *fname, int *errcode,
     }
 
     /*
-     * Some reduced MIME handling. Only use if boundary == NULL.
+     * Some reduced MIME handling. Only use if boundary == NULL. Also
+     * accept the "X-Orcl-Content-Type" used by some braindead program.
      */
     if (boundary == NULL && !ismime) {
-      if (_FP_strnicmp (line, "Content-Type", 12) == 0) {
+      if (_FP_strnicmp (line, "Content-Type", 12) == 0 ||
+	  _FP_strnicmp (line, "X-Orcl-Content-Type", 19) == 0) {
 	/*
 	 * We use Content-Type to mark a new attachment and split the file.
 	 * However, we do not split if we haven't found anything encoded yet.
@@ -707,10 +727,11 @@ ScanData (FILE *datei, char *fname, int *errcode,
 	  ptr++;
 	  while (isspace (*ptr)) ptr++; p2 = ptr;
 	  while (!isspace (*p2) && *p2 != ';') p2++;
-	  _FP_free (result->mimetype);
 	  c = *p2; *p2 = '\0';
-	  if (result->mimetype == NULL)
+	  if (p2 != ptr) {
+	    _FP_free (result->mimetype);
 	    result->mimetype = _FP_strdup (ptr);
+	  }
 	  *p2 = c;
 	}
 	ctline=0;
@@ -804,42 +825,23 @@ ScanData (FILE *datei, char *fname, int *errcode,
       /*
        * Netscape-Repair code is the same as in uunconc.c
        */
-      if ((vflag = UUValidData (line, 0, &bhflag)) == 0) {
-	nflag = UUBrokenByNetscape (line);
 
-	while (vflag == 0 && nflag) {
-	  if (nflag == 1) {
-	    ptr = line + strlen (line);
-	    while (ptr>line && (*(ptr-1)=='\015' || *(ptr-1)=='\012'))
-	      ptr--;
-	    if (_FP_fgets (ptr, 255-(ptr-line), datei) == NULL)
-	      break;
-	  }
-	  else {                       /* don't need next line to repair */
-	  }
-	  if (UUNetscapeCollapse (line)) {
-	    if ((vflag = UUValidData (line, 0, &bhflag)) == 0)
-	      nflag = UUBrokenByNetscape (line);
-	  }
-	  else
-	    nflag = 0;
-	}
-	/*
-	 * Sometimes, a line is garbled even without it being split into
-	 * the next line. Then we try this in our despair
-	 */
-	if (vflag == 0) {
-	  if (UUNetscapeCollapse (line))
-	    vflag = UUValidData (line, 0, &bhflag);
-	}
-      }
+      if ((vflag = UUValidData (line, 0, &bhflag)) == 0)
+	vflag = UURepairData (datei, line, 0, &bhflag);
+
       /*
-       * Base64 and XXencoded files are very hard to distinguish (their
-       * alphabets differ in only one character). If we had a 'begin'
-       * line, this line should be XX
+       * In a few cases, we might mistake uu or xx-encoded data for
+       * Base64, because their alphabets overlap (XX and Base64 differ
+       * in only one character) and the Base64 check is performed
+       * first. If vflag==B64 and we had a begin line, try to see if
+       * the data is consistent with XX or UU.
        */
-      if (vflag == B64ENCODED && result->begin)
-	vflag = XX_ENCODED;
+      if (vflag == B64ENCODED && result->begin) {
+	if (UUValidData (line, UU_ENCODED, &bhflag) == UU_ENCODED)
+	  vflag = UU_ENCODED;
+	else if (UUValidData (line, XX_ENCODED, &bhflag) == XX_ENCODED)
+	  vflag = XX_ENCODED;
+      }
 
       if (vflag) {
 	/*
@@ -1289,6 +1291,8 @@ ScanPart (FILE *datei, char *fname, int *errcode)
      * A partial multipart message probably has only a Content-Type
      * header but nothing else. In this case, at least simulate a
      * MIME message
+     * if mimevers is not set but there are other well-known MIME
+     * headers, don't be too picky about it.
      */
     if (sstate.envelope.ctype && sstate.envelope.mimevers==NULL  &&
 	_FP_stristr (sstate.envelope.ctype, "multipart") != NULL &&
@@ -1296,6 +1300,11 @@ ScanPart (FILE *datei, char *fname, int *errcode)
       sstate.envelope.mimevers = _FP_strdup ("1.0");
       hcount = hlcount.afternl;
     }
+    else if (sstate.envelope.mimevers==NULL && sstate.envelope.ctype &&
+	     sstate.envelope.fname && sstate.envelope.ctenc) {
+      sstate.envelope.mimevers = _FP_strdup ("1.0");
+      hcount = hlcount.afternl;
+    }
 
     if (hcount < hlcount.afternl) {
       /* not a folder after all */
@@ -2643,8 +2652,8 @@ ScanPart (FILE *datei, char *fname, int *errcode)
 	return NULL;
       }
     }
+    fseek (datei, prevpos, SEEK_SET);
   }
-  fseek (datei, prevpos, SEEK_SET);
 #endif
 
   /*
